package cn.com.ty.lift.business.contract.service;

import cn.com.ty.lift.business.capital.dao.entity.CapitalRepair;
import cn.com.ty.lift.business.capital.service.CapitalRepairService;
import cn.com.ty.lift.business.contract.dao.entity.Contracts;
import cn.com.ty.lift.business.contract.dao.entity.ContractsHistory;
import cn.com.ty.lift.business.contract.dao.entity.Payment;
import cn.com.ty.lift.business.contract.dao.entity.model.request.ContractRequest;
import cn.com.ty.lift.business.contract.dao.entity.model.response.ContractExportDto;
import cn.com.ty.lift.business.contract.dao.entity.model.response.ContractManageExportDto;
import cn.com.ty.lift.business.contract.dao.entity.model.response.ContractResponse;
import cn.com.ty.lift.business.contract.dao.mapper.ContractsMapper;
import cn.com.ty.lift.business.framework.util.MessageUtils;
import cn.com.ty.lift.business.project.dao.entity.Project;
import cn.com.ty.lift.business.project.service.ProjectService;
import cn.com.ty.lift.common.utils.DateUtils;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.poi.excel.ExcelWriter;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

import static java.math.BigDecimal.ZERO;

/**
 * @author bieao
 * @date 2019/12/07
 * @description 合同管理业务层
 */
@Service
@Slf4j
@SuppressWarnings("all")
public class ContractService extends ServiceImpl<ContractsMapper, Contracts> {

    @Resource
    private ContractsMapper contractsMapper;

    @Resource
    private ContractHistoryService contractHistoryService;

    @Resource
    private PaymentService paymentService;

    @Resource
    private ProjectService projectService;

    @Resource
    private CapitalRepairService capitalRepairService;

    /**
     * @param request 合同列表查询条件
     * @return RestResponse 合同分页列表结果
     * @description 查询合同列表
     * @date 2019/12/7 11:53 AM
     */
    public IPage<ContractResponse> list(ContractRequest request) {
        /**
         * 排序字段
         */
        request.addDesc("c.end_date");
        return contractsMapper.findByCondition(request.getPage(), request);
    }


    /**
     * @param request 新增合同数据项
     * @return 1.成功, 0.失败, 消息描述
     * @description 新增合同
     * @date 2019/12/7 3:31 PM
     */
    // TODO: 2021/1/21  合同问题
    @Transactional(rollbackFor = Exception.class)
    public RestResponse add(ContractRequest request) {
        Contracts contracts = request.getContracts();
        Long projectId = request.getProjectId();
        Integer contractType = contracts.getType();
        if (Objects.isNull(projectId)) {
            if (5 == contractType) {
                //插入大修项目
                CapitalRepair repair = new CapitalRepair();
                repair.setProjectName(request.getProjectName());
                repair.setProjectAddress(request.getProjectAddress());
                repair.setStartDate(contracts.getStarDate());
                repair.setEndDate(contracts.getEndDate());
                repair.setMtCompanyId(contracts.getMtCompanyId());
                boolean result = capitalRepairService.saveCapitalRepair(repair);
                if (!result) return RestResponse.fail(MessageUtils.get("msg.add.fail"));
                contracts.setProjectId(repair.getId());
            } else {
                //插入项目信息
                Project project = new Project();
                project.setProjectName(request.getProjectName());
                project.setAddress(request.getProjectAddress());
                project.setNum(contracts.getLiftNum());
                project.setStartDate(contracts.getStarDate());
                project.setEndDate(contracts.getEndDate());
                project.setMtCompanyId(contracts.getMtCompanyId());
                boolean pro = projectService.saveProject(project);
                if (!pro) return RestResponse.fail(MessageUtils.get("msg.add.fail"));
                contracts.setProjectId(project.getId());
            }
        } else {
            contracts.setProjectId(projectId);
        }
        if (ObjectUtil.isEmpty(contracts.getCode())) {
            //按当前日期时间戳自动生成合同编号
            String contractCode = DateUtils.generateCode();
            contracts.setCode("HT" + contractCode);
        }
        List<Payment> paymentList = request.getPaymentList();
        if (CollUtil.isNotEmpty(paymentList)) {
            //应收金额总计
            BigDecimal planMoneyTotal = paymentList.stream().map(Payment::getPlanMoney).reduce(ZERO, BigDecimal::add);
            contracts.setPlanMoneyTotal(planMoneyTotal);
        }
        //插入合同信息
        boolean result = save(contracts);
        if (!result) {
            //强制手动事务回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return RestResponse.fail(MessageUtils.get("msg.add.fail"));
        }
        //批量插入收款信息
        boolean ret = paymentService.insertBatch(paymentList, contracts);
        if (!ret) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return RestResponse.fail(MessageUtils.get("msg.add.batch.fail"));
        }
        return RestResponse.success(null, MessageUtils.get("msg.add.success"));
    }

    /**
     * @param request 续签合同数据项
     * @return 1.成功, 0.失败, 消息描述
     * @description 续签合同
     * @date 2020/5/3 10:48 上午
     */
    @Transactional(rollbackFor = Exception.class)
    public RestResponse renew(ContractRequest request) {
        Contracts contracts = request.getContracts();
        //查询原有合同是否存在
        Long contractId = request.getContractId();
        Contracts oldContract = getOne(contractId);
        if (Objects.isNull(oldContract))
            return RestResponse.fail(MessageUtils.get("msg.contract.not.exist"));
        if (ObjectUtil.isEmpty(contracts.getCode())) {
            //按当前日期时间戳自动生成合同编号
            String contractCode = DateUtils.generateCode();
            contracts.setCode("HT" + contractCode);
        }
        contracts.setProjectId(request.getProjectId());
        List<Payment> paymentList = request.getPaymentList();
        if (CollUtil.isNotEmpty(paymentList)) {
            //应收金额总计
            BigDecimal planMoneyTotal = paymentList.stream().map(Payment::getPlanMoney).reduce(ZERO, BigDecimal::add);
            contracts.setPlanMoneyTotal(planMoneyTotal);
        }
        //插入新合同信息
        boolean result = save(contracts);
        if (!result)
            return RestResponse.fail(MessageUtils.get("msg.add.fail"));

        //修改原有合同
        oldContract.setNextId(contracts.getId());
        boolean ret = updateById(oldContract);
        if (!ret) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return RestResponse.fail(MessageUtils.get("msg.add.fail"));
        }
        //查询原有付款列表
        List<Payment> oldPaymentList = paymentService.paymentList(contractId);
        paymentList.forEach(entry -> {
            entry.setContractsId(contracts.getId());
            entry.setCode("PA" + DateUtils.generateCode());
            entry.setType(contracts.getType());
        });
        //修改或删除付款列表
        return paymentService.modifyPayment(oldPaymentList, paymentList);
    }

    /**
     * @param request 合同复核
     * @return 1.成功, 0.失败, 消息描述
     * @description 合同复核
     * @date 2020/2/11 12:55 下午
     */
    public RestResponse check(ContractRequest request) {
        Contracts contracts = getOne(request.getId());
        if (Objects.isNull(contracts))
            return RestResponse.fail(MessageUtils.get("msg.contract.not.exist"));
        contracts.setIsCheck(1);
        boolean result = updateById(contracts);
        if (!result)
            return RestResponse.fail(MessageUtils.get("msg.modify.fail"));
        return RestResponse.success(MessageUtils.get("msg.modify.success"));
    }

    /**
     * @param id 合同id
     * @return 合同
     * @description 查询合同
     * @date 2019/12/20 11:04 AM
     */
    public Contracts getOne(Long id) {
        QueryWrapper<Contracts> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<Contracts> lambdaQueryWrapper = queryWrapper.lambda();
        lambdaQueryWrapper.eq(Contracts::getId, id);
        return getOne(lambdaQueryWrapper);
    }

    /**
     * @param request 修改合同、项目、付款信息数据项
     * @return 1.成功, 0.失败, 消息描述
     * @description 合同变更
     * @date 2019/12/7 3:31 PM
     */
    @Transactional(rollbackFor = Exception.class)
    public RestResponse modify(ContractRequest request) {
        Contracts contracts = request.getContracts();
        Long contractId = contracts.getId();
        Contracts oldContract = getOne(contractId);
        if (Objects.isNull(oldContract))
            return RestResponse.fail(MessageUtils.get("msg.contract.not.exist"));
        //修改合同信息
        contracts.setIsCheck(0);
        List<Payment> paymentList = request.getPaymentList();
        //应收金额总计
        BigDecimal planMoneyTotal = paymentList.stream().map(Payment::getPlanMoney).reduce(ZERO, BigDecimal::add);
        contracts.setPlanMoneyTotal(planMoneyTotal);

        boolean contractResult = updateById(contracts);
        if (!contractResult)
            return RestResponse.fail(MessageUtils.get("msg.modify.fail"));

        //生成合同操作记录
        ContractsHistory history = contractHistoryService.createOperaHistory(oldContract, contracts);
        if (Objects.nonNull(history)) {
            //保存合同操作记录
            boolean historyResult = contractHistoryService.save(history);
            if (!historyResult) {
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return RestResponse.fail(MessageUtils.get("msg.modify.fail"));
            }
        }
        //查询原有付款列表
        List<Payment> oldPaymentList = paymentService.paymentList(contractId);
        //设置合同付款的合同id
        paymentList.forEach(entry -> {
            entry.setContractsId(contracts.getId());
            entry.setCode("PA" + DateUtils.generateCode());
            entry.setType(contracts.getType());
        });
        //修改或删除付款列表
        return paymentService.modifyPayment(oldPaymentList, paymentList);
    }

    /**
     * @param id 主键id
     * @return Contracts 合同详情
     * @description 查看合同详情
     * @date 2019/12/7 11:50 AM
     */
    public Map<String, Object> detail(Long id) {
        QueryWrapper<Contracts> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("id", id);
        Contracts contracts = getOne(queryWrapper);
        List<Payment> paymentMap = paymentService.paymentList(id);
        Map<String, Object> resultMap = new HashMap<>();
        resultMap.put("contract", contracts);
        resultMap.put("paymentList", paymentMap);
        Optional<Project> project = projectService.getOne(contracts.getProjectId());
        project.ifPresent(entry -> resultMap.put("project", project.get()));
        return resultMap;
    }

    /**
     * 合同导出操作
     *
     * @param contractRequest :合同导出时的参数筛选部分
     * @return
     */
    public void export(ContractRequest contractRequest, HttpServletResponse response) {
        /**
         * 导出数据的集合
         */
        Map<String, String> headerAlias = new LinkedHashMap<>();

        /**
         * 收款管理数据表头
         */
        headerAlias.put("contractCode", "合同编号");
        headerAlias.put("memberName", "客户名称");
        headerAlias.put("payType", "收款类型");
        headerAlias.put("payCode", "收款编号");
        headerAlias.put("projectName", "项目名称");
        headerAlias.put("projectContent", "项目内容");
        headerAlias.put("nowMoney", "本期应收");
        headerAlias.put("moneys", "合同金额");
        headerAlias.put("planDate", "应收日期");
        headerAlias.put("periods", "期数");
        headerAlias.put("collectionMan", "收款责任人");
        headerAlias.put("regionName", "区域");
        headerAlias.put("paymentDays", "帐龄");
        headerAlias.put("amountInvoice", "开票金额");
        headerAlias.put("billingDate", "开票时间");
        headerAlias.put("invoiceType", "发票类型");
        headerAlias.put("workDate", "到账日期");
        headerAlias.put("workMoney", "到账金额");

        /**
         * 维保和大修数据
         */
        List<ContractExportDto> contractAllList = contractsMapper.findAllContracts(contractRequest);

        /**
         * 临修数据
         */
        List<ContractExportDto> contractTemList = contractsMapper.findAllErContracts(contractRequest);


        /**
         * 维保/大修合同管理数据
         */
        List<ContractManageExportDto> findAllContractsManageData = contractsMapper.findAllContractsManageData(contractRequest);

        /**
         * 急修数据
         */
        List<ContractManageExportDto> findAllErContractsManageData = contractsMapper.findAllErContractsManageData(contractRequest);

        /**
         * 数据合并
         */
        findAllContractsManageData.addAll(findAllErContractsManageData);

        /**
         * 数据合并
         */
        contractAllList.addAll(contractTemList);

        /**
         * 生成excel文件
         */
        ExcelWriter writer = new ExcelWriter(true, "收款管理");
        writer.setHeaderAlias(headerAlias);
        writer.write(contractAllList, true);
        writer.autoSizeColumnAll();
        writer.setSheet("合同管理");
        Map<String, String> headerAliasE = new LinkedHashMap<>();
        headerAliasE.put("projectName", "项目");
        headerAliasE.put("memberName", "客户名称");
        headerAliasE.put("regionName", "区域");
        headerAliasE.put("contractCode", "合同编号");
        headerAliasE.put("source", "来源");
        headerAliasE.put("type", "合同类型");
        headerAliasE.put("starDate", "开始时间");
        headerAliasE.put("endDate", "结束时间");
        headerAliasE.put("liftNum", "台量");
        headerAliasE.put("moneys", "金额");
        headerAliasE.put("givenDate", "签订时间");
        headerAliasE.put("givenUser", "签订人");
        headerAliasE.put("workMoney", "实收金额");
        headerAliasE.put("NoPayMoney", "未收金额");
        headerAliasE.put("partsRules", "配件条款");
        writer.setHeaderAlias(headerAliasE);
        writer.write(findAllContractsManageData, true);
        writer.autoSizeColumnAll();
        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet;charset=utf-8");
        String fileName = DateUtils.generateCode() + ".xlsx";
        response.setHeader("Content-Disposition", "attachment;filename=" + fileName);
        try {
            writer.flush(response.getOutputStream());
            // 关闭writer，释放内存
            writer.close();
        } catch (IOException e) {
            e.printStackTrace();
            log.error("合同数据数据导出失败", e);
        }
    }

    public List<Map<String, Object>> getIds(Long mtCompanyId) {

        //查询项目结果集
        List<Project> projects = projectService.getBaseMapper().selectList(
                new QueryWrapper<Project>()
                        .lambda()
                        .eq(Project::getMtCompanyId, mtCompanyId)
                        //1.尚未开始 2:服务中 4.逾期
                        .in(Project::getProjectStatus, 1, 2, 4)
                        .select(Project::getId, Project::getProjectName, Project::getAddress)
        );
        //查询大修结果集
        List<CapitalRepair> capitalRepairs = capitalRepairService.getBaseMapper().selectList(
                new QueryWrapper<CapitalRepair>()
                        .lambda()
                        .eq(CapitalRepair::getMtCompanyId, mtCompanyId)
                        //1:尚未开始
                        .eq(CapitalRepair::getProjectStatus, 1)
                        .select(CapitalRepair::getId, CapitalRepair::getProjectName, CapitalRepair::getProjectAddress)
        );

        ArrayList<Map<String, Object>> mapArrayList = new ArrayList<>();

        for (Project project : projects) {
            Map<String, Object> map = new HashMap() {{
                put("id", project.getId().toString());
                put("name", project.getProjectName());
                put("address", project.getAddress());
                put("isCapitalRepair", false);
            }};
            mapArrayList.add(map);
        }
        for (CapitalRepair capitalRepair : capitalRepairs) {
            Map<String, Object> map = new HashMap() {{
                put("id", capitalRepair.getId().toString());
                put("name", capitalRepair.getProjectName());
                put("address", capitalRepair.getProjectAddress());
                put("isCapitalRepair", true);
            }};
            mapArrayList.add(map);
        }

        return mapArrayList;
    }

    public List<Project> getProjects(Long mtCompanyId) {

        //查询当前维保公司下的维保项目的合同列表
        List<Long> list = contractsMapper.selectList(
                new QueryWrapper<Contracts>()
                        .lambda()
                        .eq(Contracts::getMtCompanyId, mtCompanyId)
                        .ne(Contracts::getType, 5)
                        .select(Contracts::getProjectId)).stream().map(n -> n.getProjectId()).collect(Collectors.toList());
        //查询不包含合同的项目
        List<Project> projects = projectService.getBaseMapper().selectList(
                new QueryWrapper<Project>()
                        .lambda()
                        .eq(Project::getMtCompanyId, mtCompanyId)
                        .in(Project::getProjectStatus, 1, 2, 4)
                        .notIn(Project::getId, list)
        );

        return projects;

    }

    public List<CapitalRepair> getCapitalRepairs(Long mtCompanyId) {

        //查询当前维保公司下的维保项目的合同列表
        List<Long> list = contractsMapper.selectList(
                new QueryWrapper<Contracts>()
                        .lambda()
                        .eq(Contracts::getMtCompanyId, mtCompanyId)
                        .eq(Contracts::getType, 5)
                        .select(Contracts::getProjectId)).stream().map(n -> n.getProjectId()).collect(Collectors.toList());

        //查询不包含合同的大修项目
        List<CapitalRepair> capitalRepairs = capitalRepairService.getBaseMapper().selectList(
                new QueryWrapper<CapitalRepair>()
                        .lambda()
                        .eq(CapitalRepair::getMtCompanyId, mtCompanyId)
                        .eq(CapitalRepair::getProjectStatus, 1)
                        .notIn(CapitalRepair::getId, list)
        );

        return capitalRepairs;
    }
}
